#!/usr/bin/env python

import io
import sys

from PIL import Image
from PyQt5 import QtCore
from PyQt5.QtMultimedia import *
from PyQt5.QtMultimediaWidgets import *
from PyQt5.QtWidgets import *
from pyzbar import pyzbar


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()

        self.available_cameras = QCameraInfo.availableCameras()
        if not self.available_cameras:
            print("No camera found.")
            sys.exit()

        self.camera = QCamera(self.available_cameras[0])
        self.camera.setCaptureMode(QCamera.CaptureStillImage)
        self.camera.start()
        self.current_camera_name = self.available_cameras[0].description()
        self.capture = QCameraImageCapture(self.camera)
        self.capture.setCaptureDestination(QCameraImageCapture.CaptureToBuffer)
        self.capture.imageAvailable.connect(self.cameraReceiver)

        self.status = QStatusBar()
        self.setStatusBar(self.status)
        self.status.showMessage(f"Using {self.current_camera_name}")
        self.viewfinder = QCameraViewfinder()
        self.viewfinder.show()
        self.setCentralWidget(self.viewfinder)
        self.camera.setViewfinder(self.viewfinder)
        toolbar = QToolBar()
        self.addToolBar(toolbar)
        recognize_action = QAction("Recognize Code", self)
        recognize_action.triggered.connect(self.recognize)
        toolbar.addAction(recognize_action)
        self.setGeometry(100, 100, 800, 600)
        self.show()

    def recognize(self):
        self.camera.searchAndLock()
        self.capture.capture()
        self.camera.unlock()

    def cameraReceiver(self, requestId, frame):
        if frame.isValid() and frame.map(QAbstractVideoBuffer.MapMode.ReadOnly):
            # Convert QImage to pil
            buffer = QtCore.QBuffer()
            buffer.open(QtCore.QBuffer.ReadWrite)
            frame.image().save(buffer, "JPEG")
            pil_im = Image.open(io.BytesIO(buffer.data()))
            barcodes = pyzbar.decode(pil_im)
            for barcode in barcodes:
                data = barcode.data.decode('utf-8')
                print(f"Found Type : {barcode.type} Barcode : {data}")
        frame.unmap()


if __name__ == "__main__":
    app = QApplication(sys.argv)
    mainWindow = MainWindow()
    mainWindow.show()
    sys.exit(app.exec_())
